##############################################################################
#
# Copyright (c) 2003 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Container constraint tests

$Id: test_constraints.py 95341 2009-01-28 15:59:18Z wosc $
"""
import unittest
from zope.testing import doctest, module

def setUp(test):
    module.setUp(test, 'zope.container.constraints_txt')

def tearDown(test):
    module.tearDown(test, 'zope.container.constraints_txt')

def test_suite():
    return unittest.TestSuite((
        doctest.DocTestSuite('zope.container.constraints'),
        doctest.DocFileSuite('../constraints.txt',
                             setUp=setUp, tearDown=tearDown),
        ))

if __name__ == '__main__': unittest.main()
